@@ -7,7 +7,7 @@ annotations made on Shakespeares World.
 from .extractor_wrapper import extractor_wrapper
 
 
-@extractor_wrapper
+@extractor_wrapper()
 def sw_variant_extractor(classification, **kwargs):
     '''Extract all variants in a classification into one list
 
