@@ -9,17 +9,34 @@ from collections import OrderedDict, Counter
 import copy
 import numpy as np
 from sklearn.cluster import DBSCAN
+import re
 
 
 def tokenize(self, contents):
     '''Tokenize only on space so angle bracket tags are not split'''
-    return contents.split()
+    return re.findall(r'[^\s$]+|[$]', contents)
 
 
 # override the built-in tokenize
 col.core_classes.WordPunctuationTokenizer.tokenize = tokenize
 
 
+def extractor_index(x):
+    edx = []
+    i_prev = None
+    e_prev = -1
+    for i in x:
+        if i_prev is None:
+            i_prev = i
+        else:
+            if i != i_prev:
+                i_prev = i
+                e_prev = -1
+        e_prev += 1
+        edx.append(e_prev)
+    return edx
+
+
 def overlap(x, y, tol=0):
     '''Check if two line segments overlap
 
@@ -69,7 +86,7 @@ def gutter(lines_in, tol=0):
                 if o_lines.any():
                     comp = np.vstack([overlap_lines[o_lines], l])
                     overlap_lines[o_lines] = [comp.min(), comp.max()]
-                    overlap_lines = np.vstack({tuple(row) for row in overlap_lines})
+                    overlap_lines = np.vstack(list({tuple(row) for row in overlap_lines}))
                 else:
                     overlap_lines = np.vstack([overlap_lines, l])
         overlap_lines.sort(axis=0)
@@ -168,16 +185,26 @@ def consensus_score(clusters_text):
     -------
     consensus_score : float
         A value indicating the average number of users that agree on the line of text.
+    consensus_text : str
+        A string with the consensus sentence
     '''
     text_filter = [list(filter(('').__ne__, text)) for text in clusters_text if text]
-    max_counts = [Counter(text).most_common(1)[0][1] for text in text_filter if text]
-    if len(max_counts) == 0:
-        return 0.0
+    most_common = [Counter(text).most_common(1)[0] for text in text_filter if text]
+    if len(most_common) == 0:
+        return 0.0, ''
     else:
-        return sum(max_counts) / len(max_counts)
+        common_text, max_counts = zip(*most_common)
+        return sum(max_counts) / len(max_counts), ' '.join(common_text)
 
 
-def cluster_by_word(word_line, xy_line, text_line, annotation_labels, kwargs_cluster, kwargs_dbscan):
+def cluster_by_word(
+    word_line,
+    xy_line,
+    text_line,
+    annotation_labels,
+    kwargs_cluster,
+    kwargs_dbscan
+):
     '''A function to take the annotations for one line of text and cluster them
     based on the words in the line.
 
@@ -192,7 +219,7 @@ def cluster_by_word(word_line, xy_line, text_line, annotation_labels, kwargs_clu
     annotation_labels : np.array
         An nx1 array with a lable indicating what annotaiton each word belongs to.
     kwargs_cluster : dict
-        A dictionary containing the `eps_*`, `metric`, and `dot_freq` keywords
+        A dictionary containing the `eps_*` and `dot_freq` keywords
     kwargs_dbscan : dict
         A dictionary containing all the other DBSCAN keywords
 
@@ -207,7 +234,7 @@ def cluster_by_word(word_line, xy_line, text_line, annotation_labels, kwargs_clu
         list per cluster. Note: the empty strings that were added to each annotaiton are
         stripped before returning the words.
     '''
-    db_words = DBSCAN(eps=kwargs_cluster['eps_word'], metric=kwargs_cluster['metric'], **kwargs_dbscan).fit(word_line)
+    db_words = DBSCAN(eps=kwargs_cluster['eps_word'], **kwargs_dbscan).fit(word_line)
     word_labels = sort_labels(db_words.labels_, word_line)
     clusters_x = []
     clusters_y = []
@@ -227,7 +254,13 @@ def cluster_by_word(word_line, xy_line, text_line, annotation_labels, kwargs_clu
     return clusters_x, clusters_y, clusters_text
 
 
-def align_words(word_line, xy_line, text_line, kwargs_cluster, kwargs_dbscan):
+def align_words(
+    word_line,
+    xy_line,
+    text_line,
+    kwargs_cluster,
+    kwargs_dbscan
+):
     '''A function to take the annotations for one line of text, aligns the words,
     and finds the end-points for the line.
 
@@ -239,8 +272,10 @@ def align_words(word_line, xy_line, text_line, kwargs_cluster, kwargs_dbscan):
         An nx2 array with the non-rotated (x, y) positions of each dot.
     text_line : np.array
         An nx1 array with the text for each dot.
+    gs_line : np.array
+        An array of bools indicating if the annotation was made in gold standard mode
     kwargs_cluster : dict
-        A dictionary containing the `eps_*`, `metric`, and `dot_freq` keywords
+        A dictionary containing the `eps_*` and `dot_freq` keywords
     kwargs_dbscan : dict
         A dictionary containing all the other DBSCAN keywords
 
@@ -260,7 +295,7 @@ def align_words(word_line, xy_line, text_line, kwargs_cluster, kwargs_dbscan):
     clusters_text = []
     # ignore min_samples when trying to find the end points of a line
     min_samples = kwargs_dbscan.pop('min_samples', 1)
-    db_words = DBSCAN(eps=kwargs_cluster['eps_word'], metric=kwargs_cluster['metric'], min_samples=1, **kwargs_dbscan).fit(word_line)
+    db_words = DBSCAN(eps=kwargs_cluster['eps_word'], min_samples=1, **kwargs_dbscan).fit(word_line)
     # put min_samples back in
     kwargs_dbscan['min_samples'] = min_samples
     word_labels = sort_labels(db_words.labels_, word_line)
@@ -279,8 +314,7 @@ def align_words(word_line, xy_line, text_line, kwargs_cluster, kwargs_dbscan):
                 collation.add_plain_witness(key, t)
                 witness_key.append(key)
         if len(collation.witnesses) > 0:
-            scheduler = col.near_matching.Scheduler()
-            alignment_table = col.collate(collation, near_match=True, segmentation=False, scheduler=scheduler)
+            alignment_table = col.collate(collation, near_match=True, segmentation=False)
             for cols in alignment_table.columns:
                 word_dict = cols.tokens_per_witness
                 word_list = []
@@ -290,13 +324,22 @@ def align_words(word_line, xy_line, text_line, kwargs_cluster, kwargs_dbscan):
                     else:
                         word_list.append('')
                 clusters_text.append(word_list)
-            # fix memory leak by deleting these
-            del scheduler
+            # fix memory leak by deleting this
             del alignment_table
     return clusters_x, clusters_y, clusters_text
 
 
-def cluster_by_line(xy_rotate, xy_gutter, text_gutter, annotation_labels, kwargs_cluster, kwargs_dbscan):
+def cluster_by_line(
+    xy_rotate,
+    xy_gutter,
+    text_gutter,
+    annotation_labels,
+    gs_gutter,
+    data_index_gutter,
+    ext_index_gutter,
+    kwargs_cluster,
+    kwargs_dbscan
+):
     '''A function to take the annotations for one `slope_label` and cluster them
     based on perpendicular distance (e.g. lines of text).
 
@@ -315,8 +358,14 @@ def cluster_by_line(xy_rotate, xy_gutter, text_gutter, annotation_labels, kwargs
         An array of shape nx1 containing a unique lable indicating what annotation
         each position/text came from.  This information is used to ensure one annotation
         does not span multiple lines.
+    gs_gutter : np.array
+        An array of bools indicating if the annotation was made in gold standard mode
+    data_index_gutter : np.array
+        An array of indicies indicating what calssification each classification came from
+    ext_index_gutter : np.array
+        A list of extractor indicies used to map the reduction to the extract
     kwargs_cluster : dict
-        A dictionary containing the `eps_*`, `metric`, and `dot_freq` keywords
+        A dictionary containing the `eps_*`, and `dot_freq` keywords
     kwargs_dbscan : dict
         A dictionary containing all the other DBSCAN keywords
 
@@ -332,7 +381,7 @@ def cluster_by_line(xy_rotate, xy_gutter, text_gutter, annotation_labels, kwargs
     words = xy_rotate[:, 0].reshape(-1, 1)
     a_lables = np.unique(annotation_labels)
     avg_lines = np.array([lines[annotation_labels == a].mean() for a in a_lables]).reshape(-1, 1)
-    db_lines = DBSCAN(eps=kwargs_cluster['eps_line'], metric=kwargs_cluster['metric'], **kwargs_dbscan).fit(avg_lines)
+    db_lines = DBSCAN(eps=kwargs_cluster['eps_line'], **kwargs_dbscan).fit(avg_lines)
     line_labels = sort_labels(db_lines.labels_, avg_lines)
     frame_lines = []
     for line_label in line_labels:
@@ -342,41 +391,74 @@ def cluster_by_line(xy_rotate, xy_gutter, text_gutter, annotation_labels, kwargs
         for a_label in a_lables[ldx]:
             adx |= annotation_labels == a_label
         if kwargs_cluster['dot_freq'] == 'word':
-            clusters_x, clusters_y, clusters_text = cluster_by_word(words[adx], xy_gutter[adx], text_gutter[adx], annotation_labels[adx], kwargs_cluster, kwargs_dbscan)
+            clusters_x, clusters_y, clusters_text = cluster_by_word(
+                words[adx],
+                xy_gutter[adx],
+                text_gutter[adx],
+                annotation_labels[adx],
+                kwargs_cluster,
+                kwargs_dbscan
+            )
         elif kwargs_cluster['dot_freq'] == 'line':
-            clusters_x, clusters_y, clusters_text = align_words(words[adx], xy_gutter[adx], text_gutter[adx], kwargs_cluster, kwargs_dbscan)
+            clusters_x, clusters_y, clusters_text = align_words(
+                words[adx],
+                xy_gutter[adx],
+                text_gutter[adx],
+                kwargs_cluster,
+                kwargs_dbscan
+            )
+        consensus_score_value, consensus_text = consensus_score(clusters_text)
         line_dict = {
             'clusters_x': clusters_x,
             'clusters_y': clusters_y,
             'clusters_text': clusters_text,
             'number_views': ldx.sum(),
-            'consensus_score': consensus_score(clusters_text),
+            'consensus_score': consensus_score_value,
+            'consensus_text': consensus_text,
             'line_slope': float(kwargs_cluster['avg_slope']),
             'slope_label': int(kwargs_cluster['slope_label']),
-            'gutter_label': int(kwargs_cluster['gutter_label'])
+            'gutter_label': int(kwargs_cluster['gutter_label']),
+            'gold_standard': gs_gutter[ldx].tolist(),
+            'data_index': data_index_gutter[ldx].tolist(),
+            'extract_index': ext_index_gutter[ldx].tolist()
         }
         if len(line_dict['clusters_x']) > 0:
             frame_lines.append(line_dict)
     return frame_lines
 
 
-def cluster_by_gutter(x_slope, y_slope, text_slope, kwargs_cluster, kwargs_dbscan):
+def cluster_by_gutter(
+    x_slope,
+    y_slope,
+    text_slope,
+    gs_slope,
+    data_index_slope,
+    ext_index_slope,
+    kwargs_cluster,
+    kwargs_dbscan
+):
     '''A function to take the annotations for each frame of a subject and group them
     based on what side of the page gutter they are on.
 
     Parameters
     ----------
-    x_slope : list
+    x_slope : np.array
         A list-of-lists of the x values for each drawn dot. There is one item in the
         list for annotation made by the user.
-    y_slope : list
+    y_slope : np.array
         A list-of-lists of the y values for each drawn dot. There is one item in the
         list for annotation made by the user.
-    text_slope : list
+    text_slope : np.array
         A list-of-lists of the text for each drawn dot. There is one item in the
         list for annotation made by the user.
+    gs_slope : np.array
+        A list of bools indicating if the annotation was made in gold standard mode
+    data_index_slope : np.array
+        A list of indicies indicating what calssification each classification came from
+    ext_index_slope : np.array
+        A list of extractor indicies used to map the reduction to the extract
     kwargs_cluster : dict
-        A dictionary containing the `eps_*`, `metric`, and `dot_freq` keywords
+        A dictionary containing the `eps_*` and `dot_freq` keywords
     kwargs_dbscan : dict
         A dictionary containing all the other DBSCAN keywords
 
@@ -399,32 +481,58 @@ def cluster_by_gutter(x_slope, y_slope, text_slope, kwargs_cluster, kwargs_dbsca
         xy_gutter = np.array(list(zip(np.hstack(x_slope[gdx]), np.hstack(y_slope[gdx]))))
         text_gutter = np.hstack(text_slope[gdx])
         kwargs_cluster['gutter_label'] = gutter_label
-        frame_lines = cluster_by_line(xy_rotate, xy_gutter, text_gutter, annotation_label, kwargs_cluster, kwargs_dbscan)
+        frame_lines = cluster_by_line(
+            xy_rotate,
+            xy_gutter,
+            text_gutter,
+            annotation_label,
+            gs_slope[gdx],
+            data_index_slope[gdx],
+            ext_index_slope[gdx],
+            kwargs_cluster,
+            kwargs_dbscan
+        )
         frame_gutter += frame_lines
     return frame_gutter
 
 
-def cluster_by_slope(x_frame, y_frame, text_frame, slope_frame, kwargs_cluster, kwargs_dbscan):
+def cluster_by_slope(
+    x_frame,
+    y_frame,
+    text_frame,
+    slope_frame,
+    gs_frame,
+    data_index_frame,
+    ext_index_frame,
+    kwargs_cluster,
+    kwargs_dbscan
+):
     '''A function to take the annotations for one `gutter_label` and cluster them
     based on what slope the transcription is.
 
     Parameters
     ----------
-    x_frame : list
+    x_frame : np.array
         A list-of-lists of the x values for each drawn dot. There is one item in the
         list for annotation made by the user.
-    y_frame : list
+    y_frame : np.array
         A list-of-lists of the y values for each drawn dot. There is one item in the
         list for annotation made by the user.
-    text_frame : list
+    text_frame : np.array
         A list-of-lists of the text for each drawn dot. There is one item in the
         list for annotation made by the user. The inner text lists are padded with
         an empty string at the end so there is the same number of words as there are
         dots.
-    slope_frame : list
+    slope_frame : np.array
         A list of the slopes (in deg) for each annotation
+    gs_frame : np.array
+        A list of bools indicating if the annotation was made in gold standard mode
+    data_index_frame : np.array
+        A list of indicies indicating what calssification each classification came from
+    ext_index_frame : np.array
+        A list of extractor indicies used to map the reduction to the extract
     kwargs_cluster : dict
-        A dictionary containing the `eps_*`, `metric`, and `dot_freq` keywords
+        A dictionary containing the `eps_*` and `dot_freq` keywords
     kwargs_dbscan : dict
         A dictionary containing all the other DBSCAN keywords
 
@@ -442,23 +550,67 @@ def cluster_by_slope(x_frame, y_frame, text_frame, slope_frame, kwargs_cluster,
             avg_slope = avg_angle(slope_frame[sdx])
             kwargs_cluster['avg_slope'] = avg_slope
             kwargs_cluster['slope_label'] = slope_label
-            frame_gutter = cluster_by_gutter(x_frame[sdx], y_frame[sdx], text_frame[sdx], kwargs_cluster, kwargs_dbscan)
+            frame_gutter = cluster_by_gutter(
+                x_frame[sdx],
+                y_frame[sdx],
+                text_frame[sdx],
+                gs_frame[sdx],
+                data_index_frame[sdx],
+                ext_index_frame[sdx],
+                kwargs_cluster,
+                kwargs_dbscan
+            )
             frame_slope += frame_gutter
     return frame_slope
 
 
-def cluster_by_frame(data_by_frame, kwargs_cluster, kwargs_dbscan):
+def cluster_by_frame(
+    data_by_frame,
+    kwargs_cluster,
+    kwargs_dbscan,
+    user_ids_input,
+    low_consensus_threshold
+):
     reduced_data = OrderedDict()
+    low_consensus_lines = 0
+    number_of_lines = 0
     for frame, value in data_by_frame.items():
         reduced_data[frame] = []
+        gs_frame = np.array(copy.deepcopy(value['gold_standard']))
+        data_index_frame = np.array(copy.deepcopy(value['data_index']))
         slope_frame = np.array(copy.deepcopy(value['slope'])).reshape(-1, 1)
         x_frame = np.array(copy.deepcopy(value['x']))
         y_frame = np.array(copy.deepcopy(value['y']))
         text_frame = copy.deepcopy(value['text'])
+        ext_index_frame = np.array(extractor_index(value['data_index']))
         # pad with empty strings to keep array sizes the same
         for t in text_frame:
             t.append('')
         text_frame = np.array(text_frame)
-        frame_slope = cluster_by_slope(x_frame, y_frame, text_frame, slope_frame, kwargs_cluster, kwargs_dbscan)
+        frame_slope = cluster_by_slope(
+            x_frame,
+            y_frame,
+            text_frame,
+            slope_frame,
+            gs_frame,
+            data_index_frame,
+            ext_index_frame,
+            kwargs_cluster,
+            kwargs_dbscan
+        )
+        number_of_lines += len(frame_slope)
+        for line in frame_slope:
+            data_index = line.pop('data_index')
+            line['user_ids'] = [user_ids_input[di] for di in data_index]
+            if line['consensus_score'] < low_consensus_threshold:
+                line['low_consensus'] = True
+                line['flagged'] = True
+                low_consensus_lines += 1
+            else:
+                line['low_consensus'] = False
+                line['flagged'] = False
         reduced_data[frame] += frame_slope
+    reduced_data['low_consensus_lines'] = low_consensus_lines
+    reduced_data['transcribed_lines'] = number_of_lines
+    reduced_data['reducer'] = 'poly_line_text_reducer'
     return reduced_data
